/*
  Filename:	main-curses.c
  Author:	Ron Burkey
  License:	This stuff was all just cut-and-pasted from the "NCURSES
  		Programming HOTWO" by Pradeep Padala, whose license seems to
		be "free to use for anything" except that the following
		notice must be included with any substantial reproduction
		of the software:
		
		    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY
		    OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT 
		    LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
		    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
		    IN NO EVENT SHALL THE ABOVE COPYRIGHT HOLDERS BE
		    LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, 
		    WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, 
		    ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
		    SOFTWARE OR THE USE OR OTHER DEALINGS IN THE 
		    SOFTWARE.
		
  Purpose:     	A simple curses sample program for the IMCROSS project.
  Mod history:	2009-04-17 RSB	Created.
*/

#include <stdio.h>
#include <string.h>
#include <curses.h>

int
HelloWorld (void)
{
  initscr();                      /* Start curses mode              */
  clear ();
  printw("Hello World !!!");      /* Print Hello World              */
  refresh();                      /* Print it on to the real screen */
  getch();                        /* Wait for user input */
  endwin();                       /* End curses mode                */
}

int
InitializationFunctions (void)
{
  int ch;

  initscr();                      /* Start curses mode            */
  clear ();
  raw();                          /* Line buffering disabled      */
  keypad(stdscr, TRUE);           /* We get F1, F2 etc..          */
  noecho();                       /* Don't echo() while we do getch */

  printw("Type any character to see it in bold\n");
  ch = getch();                   /* If raw() hadn't been called
				   * we have to press enter before it
				   * gets to the program          */
  if(ch == KEY_F(1))              /* Without keypad enabled this will */
	  printw("F1 Key pressed");/*  not get to us either       */
				  /* Without noecho() some ugly escape
				   * charachters might have been printed
				   * on screen                    */
  else
  {       printw("The pressed key is ");
	  attron(A_BOLD);
	  printw("%c", ch);
	  attroff(A_BOLD);
  }
  refresh();                      /* Print it on to the real screen */
  getch();                        /* Wait for user input */
  endwin();                       /* End curses mode                */

  return 0;

}

int
SimplePrintw (void)
{
  char mesg[]="Just a string";           /* message to be appeared on the screen */
  int row,col;                           /* to store the number of rows and *
					  * the number of colums of the screen */
  initscr();                             /* start the curses mode */
  clear ();
  getmaxyx(stdscr,row,col);              /* get the number of rows and columns */
  mvprintw(row/2,(col-strlen(mesg))/2,"%s",mesg);
					/* print the message at the center of the screen */
  mvprintw(row-2,0,"This screen has %d rows and %d columns\n",row,col);
  printw("Try resizing your window(if possible) and then run this program again");
  refresh();
  getch();
  endwin();

  return 0;
}

int
SimpleScanw (void)
{
  char mesg[]="Enter a string: ";        /* message to be appeared on the screen */
  char str[80];
  int row,col;                           /* to store the number of rows and *
					  * the number of colums of the screen */
  initscr();                             /* start the curses mode */
  clear ();
  getmaxyx(stdscr,row,col);              /* get the number of rows and columns */
  mvprintw(row/2,(col-strlen(mesg))/2,"%s",mesg);
				/* print the message at the center of the screen */
  getstr(str);
  mvprintw(LINES - 2, 0, "You Entered: %s", str);
  getch();
  endwin();

  return 0;
}

int
SimpleAttributes (int argc, char *argv[])
{ 
  int ch, prev, row, col;
  prev = EOF;
  FILE *fp;
  int y, x;

  if(argc != 2)
  {
    printf("Usage: %s <a c file name>\n", argv[0]);
    exit(1);
  }
  fp = fopen(argv[1], "r");
  if(fp == NULL)
  {
    perror("Cannot open input file");
    return(1);
  }
  initscr();                            /* Start curses mode */
  clear ();
  getmaxyx(stdscr, row, col);           /* find the boundaries of the screeen */
  while((ch = fgetc(fp)) != EOF)        /* read the file till we reach the end */
  {
    getyx(stdscr, y, x);                /* get the current curser position */
    if(y == (row - 1))                  /* are we are at the end of the screen */
    {
      printw("<-Press Any Key->");      /* tell the user to press a key */
      getch();
      clear();                          /* clear the screen */
      move(0, 0);                       /* start at the beginning of the screen */
    }
    if(prev == '/' && ch == '*')        /* If it is / and * then only
                                         * switch bold on */    
    {
      attron(A_BOLD);                   /* cut bold on */
      getyx(stdscr, y, x);              /* get the current curser position */
      move(y, x - 1);                   /* back up one space */
      printw("%c%c", '/', ch);          /* The actual printing is done here */
    }
    else
      printw("%c", ch);
    refresh();
    if(prev == '*' && ch == '/')
      attroff(A_BOLD);                  /* Switch it off once we got *
                                         * and then / */
    prev = ch;
  }
  getch ();
  endwin();                             /* End curses mode */
  fclose(fp);
  return 0;
}

int
ChgatUsage (void)
{       
  initscr();                      /* Start curses mode            */
  clear ();
  start_color();                  /* Start color functionality    */
  
  init_pair(1, COLOR_CYAN, COLOR_BLACK);
  printw("A Big string which i didn't care to type fully ");
  mvchgat(0, 0, -1, A_BLINK, 1, NULL);    
  /* 
   * First two parameters specify the position at which to start 
   * Third parameter number of characters to update. -1 means till 
   * end of line
   * Forth parameter is the normal attribute you wanted to give 
   * to the charcter
   * Fifth is the color index. It is the index given during init_pair()
   * use 0 if you didn't want color
   * Sixth one is always NULL 
   */
  refresh();
  getch();
  endwin();                       /* End curses mode                */
  return 0;
}

WINDOW *create_newwin(int height, int width, int starty, int startx);
void destroy_win(WINDOW *local_win);

int 
WindowBorder (void)
{       
  WINDOW *my_win;
  int startx, starty, width, height;
  int ch;

  initscr();                      /* Start curses mode            */
  clear ();
  cbreak();                       /* Line buffering disabled, Pass on
				   * everty thing to me           */
  keypad(stdscr, TRUE);           /* I need that nifty F1         */

  height = 3;
  width = 10;
  starty = (LINES - height) / 2;  /* Calculating for a center placement */
  startx = (COLS - width) / 2;    /* of the window                */
  printw("Press F1 to exit");
  refresh();
  my_win = create_newwin(height, width, starty, startx);

  while((ch = getch()) != KEY_F(1))
  {       
    switch(ch)
      {       
      case KEY_LEFT:
	destroy_win(my_win);
	my_win = create_newwin(height, width, starty,--startx);
	break;
      case KEY_RIGHT:
	destroy_win(my_win);
	my_win = create_newwin(height, width, starty,++startx);
	break;
      case KEY_UP:
	destroy_win(my_win);
	my_win = create_newwin(height, width, --starty,startx);
	break;
      case KEY_DOWN:
	destroy_win(my_win);
	my_win = create_newwin(height, width, ++starty,startx);
	break;  
      }
  }
	  
  endwin();                       /* End curses mode                */
  return 0;
}

WINDOW *create_newwin(int height, int width, int starty, int startx)
{       
  WINDOW *local_win;

  local_win = newwin(height, width, starty, startx);
  box(local_win, 0 , 0);          /* 0, 0 gives default characters 
				   * for the vertical and horizontal
				   * lines                        */
  wrefresh(local_win);            /* Show that box                */

  return local_win;
}

void destroy_win(WINDOW *local_win)
{       
  /* box(local_win, ' ', ' '); : This won't produce the desired
   * result of erasing the window. It will leave it's four corners 
   * and so an ugly remnant of window. 
   */
  wborder(local_win, ' ', ' ', ' ',' ',' ',' ',' ',' ');
  /* The parameters taken are 
   * 1. win: the window on which to operate
   * 2. ls: character to be used for the left side of the window 
   * 3. rs: character to be used for the right side of the window 
   * 4. ts: character to be used for the top side of the window 
   * 5. bs: character to be used for the bottom side of the window 
   * 6. tl: character to be used for the top left corner of the window 
   * 7. tr: character to be used for the top right corner of the window 
   * 8. bl: character to be used for the bottom left corner of the window 
   * 9. br: character to be used for the bottom right corner of the window
   */
  wrefresh(local_win);
  delwin(local_win);
}


int
main (int argc, char *argv[])
{

  HelloWorld ();
  InitializationFunctions ();
  SimplePrintw ();
  SimpleScanw ();
  SimpleAttributes (argc, argv);
  ChgatUsage ();
  WindowBorder ();
  
  return (0);
}

