/*
  Filename:	main-readline.c
  Author:	Ron Burkey
  License:	I've simply slavishly cut-and-pasted most of this from the GNU readline
  		documentation.  The license for the documentation is the GNU FDL, and I
		frankly don't know what that means regarding the licensing of the code
		snippets within.
  Purpose:     	A simple readline sample program for the IMCROSS project.
  Mod history:	2009-04-17 RSB	Created.
*/

#include <stdio.h>
#include <string.h>
#include <readline/readline.h>

/* A static variable for holding the line. */
static char *line_read = (char *)NULL;

/* Read a string, and return a pointer to it.
   Returns NULL on EOF. */
char *
rl_gets ()
{
  /* If the buffer has already been allocated,
     return the memory to the free pool. */
  if (line_read)
    {
      free (line_read);
      line_read = (char *)NULL;
    }

  /* Get a line from the user. */
  line_read = readline ("> ");

  /* If the line has any text in it,
     save it on the history. */
  if (line_read && *line_read)
    add_history (line_read);

  return (line_read);
}

#ifdef NEVER

#define STRING(x) #x
#define RL_PRINT(RL_CHAR) \
{ \
  extern char *RL_CHAR; \
  if (RL_CHAR == NULL) \
    printf ("%s = NULL\n", STRING(RL_CHAR)); \
  else \
    printf ("*%s = 0x%02X\n", STRING(RL_CHAR), *RL_CHAR); \
}

static void
DebugTest (void)
{
#ifndef __APPLE__
  RL_PRINT (_rl_term_cr);
  RL_PRINT (_rl_term_backspace);
  printf ("This gobble\b\b\b\b\b\bis a test of backspacing.\n");
  printf ("Gobble\rThis is a test of carriage-return.\n");
#endif
}

#else

#define DebugTest()

#endif

int
main (void)
{
  DebugTest ();
  printf ("\n");
  printf ("All this sample program does is to repeatedly prompt you for input.\n");
  printf ("After you input a line, it is added to the command history, and you\n");
  printf ("are prompted again.  You can use the readline library\'s commands to\n");
  printf ("navigate through the command history.  When you want to finally quit,\n");
#ifdef WIN32  
  printf ("just hit Ctrl-C a couple of times very quickly.\n");
#else
  printf ("just hit Ctrl-C.\n");
#endif  
  printf ("\n");
  while (1)
    {
      if (NULL == rl_gets ())
        printf ("(empty)\n");
      else
        {
          printf ("%s\n", line_read);
	}
      DebugTest ();
    }
  return (0);
}


// Experimental stuff for doing without curses/ncurses/PDCurses, which readline
// seems to depend upon.

